const { machine } = require('asyncmachine')
const assert = require('assert')

const example = machine({
  Requested: { add: ['Implied'] },
  Implied: {},
  Delayed: {}
})
example.id('').logLevel(3)

// SETUP TRANSITIONS

example.Requested_enter = function(name, value) {
	// target states inside of a negotiation handler are available as `this.to()`
  assert(this.to().includes('Requested'))
  assert(this.to().includes('Implied'))
  assert(name == 'john')
  assert(value == 5)
}

example.Requested_state = function(name, value) {
  assert(this.is().includes('Requested'))
  assert(this.is().includes('Implied'))
  assert(name == 'john')
  assert(value == 5)

  this.addByListener('Delayed')(name, value)
}

example.Implied_state = function(name, value) {
  assert(this.is().includes('Requested'))
  assert(this.is().includes('Implied'))
  // only the Requested states get the call params passed through
  assert(name === undefined)
  assert(value === undefined)
}

example.Delayed_state = function(name, value) {
  assert(this.is().includes('Delayed'))
  assert(this.is().includes('Requested'))
  assert(this.is().includes('Implied'))
  assert(name == 'john')
  assert(value == 5)

  // this will trigger a self transition, using the node-style callback
  this.addByCallback('Delayed')(null, name, value)
}

example.Delayed_Delayed = function(name, value) {
  assert(this.is().includes('Delayed'))
  assert(this.is().includes('Requested'))
  assert(this.is().includes('Implied'))
  assert(name == 'john')
  assert(value == 5)
}

// RUN

example.add('Requested', 'john', 5);
